import unittest
import zzpy as z


def get_now():
    import time
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


def get_today():
    return get_date(days=0, separator='-')


def get_date(days=0, separator='-'):
    import time
    return time.strftime(f"%Y{separator}%m{separator}%d", time.localtime(time.time()+days*86400))


class TestCase(unittest.TestCase):
    def test_get_date_with_no_parameters(self):
        self.assertEqual(z.get_date(), get_today())

    def test_get_date_with_days(self):
        for offset in range(-1000, 1000):
            self.assertEqual(z.get_date(offset), get_date(offset))
            self.assertEqual(z.get_date(days=offset), get_date(days=offset))
            self.assertEqual(z.get_date(offset), get_date(offset))
            self.assertEqual(z.get_date(days=offset), get_date(days=offset))

    def test_get_date_with_separator(self):
        self.assertEqual(z.get_date(), get_date(separator="-"))
        self.assertEqual(z.get_date(), z.get_date(separator="-"))
        self.assertEqual(z.get_date(separator="-"), get_date(separator="-"))
        self.assertEqual(z.get_date(separator="/"), get_date(separator="/"))

    def test_get_date_with_days_and_separator(self):
        self.assertEqual(z.get_date(), get_date(days=0, separator="-"))
        self.assertEqual(z.get_date(), z.get_date(days=0, separator="-"))
        self.assertEqual(z.get_date(days=1, separator="*"),
                         get_date(days=1, separator="*"))
        self.assertEqual(z.get_date(days=1, separator="*"),
                         z.get_date(days=1, separator="*"))
        self.assertEqual(z.get_date(days=-1, separator="*"),
                         get_date(days=-1, separator="*"))
        self.assertEqual(z.get_date(days=-1, separator="*"),
                         z.get_date(days=-1, separator="*"))



import unittest


class ZGeoTests(unittest.TestCase):
    def test_amap_geo_coding(self):
        from zzpy import amap_geo_coding
        with self.assertRaises(AssertionError):
            amap_geo_coding(address="大连市腾讯大厦")
        with self.assertRaises(AssertionError):
            amap_geo_coding(address="大连市腾讯大厦", api_key="")
        with self.assertRaises(Exception):
            amap_geo_coding(address="大连市腾讯大厦", api_key="WRONG_API_KEY")



# zprogress
from .zprogress import pb

# zconfig
from .zconfig import get_param


# zfile
from .zfile import remove_dir
from .zfile import create_dir
from .zfile import init_dir
from .zfile import get_one_file_path_from_dir
from .zfile import move_file_from_src_dir
from .zfile import get_file_line_count
from .zfile import get_file_name_list_from_dir
from .zfile import get_file_path_list_from_dir
from .zfile import read_bin_file
from .zfile import read_file
from .zfile import write_file
from .zfile import get_tmp_dir_path
from .zfile import get_work_dir_path
from .zfile import init_dirs
from .zfile import read_jsonline
from .zfile import read_jsonline_with_progressbar
from .zfile import normalize_path


# zdefinition
from .zdefinition import SystemPlatform


# zsys
from .zsys import is_windows
from .zsys import windows_else


# zmongo
from .zmongo import MongoConfig, mongo_connect


# zmysql
from .zmysql import MySQLConfig
from .zmysql import mysql_connect
from .zmysql import mysql_execute
from .zmysql import mysql_insert
from .zmysql import mysql_query
from .zmysql import mysql_query_one_value
from .zmysql import mysql_iter_table
from .zmysql import mysql_count_table


# zredis
from .zredis import redis_connect
from .zredis import redis_decode
from .zredis import ZRedis


# zgeo
from .zgeo import amap_geo_coding
from .zgeo import gcj02_to_bd09
from .zgeo import bd09_to_gcj02
from .zgeo import wgs84_to_gcj02
from .zgeo import gcj02_to_wgs84
from .zgeo import bd09_to_wgs84
from .zgeo import wgs84_to_bd09
from .zgeo import out_of_china
from .zgeo import calculate_distance


# zrandom
from .zrandom import randint
from .zrandom import randhex


# zjson
from .zjson import jsondumps


# ztime
from .ztime import get_date
from .ztime import get_today
from .ztime import get_month


# zalioss
from .zalioss import OssConfig
from .zalioss import OssFile



import unittest
import datetime
import zzpy as z


class TestCase(unittest.TestCase):
    def setUp(self):
        self.client = z.mysql_connect(url="mysql://am-8vb369k7zxdt10tz043750.zhangbei.ads.aliyuncs.com:3306/?user=amap_dba&password=sNDfwkdl6SDfw33")
    
    def tearDown(self):
        self.client.close()
        self.client = None
    
    def test_mysql_connect(self):
        client = z.mysql_connect(url="mysql://am-8vb369k7zxdt10tz043750.zhangbei.ads.aliyuncs.com:3306/?user=amap_dba&password=sNDfwkdl6SDfw33")
        self.assertIsNotNone(client)
        client.close()
        
    def test_mysql_count_table(self):
        a = z.mysql_count_table(self.client, table="sca.t_std_org")
        self.assertGreater(a, 0)
        b = z.mysql_count_table(self.client, table="sca.t_std_org", where_condition="is_deleted=0")
        self.assertGreater(b, 0)
        self.assertGreater(a, b)
    
    def test_mysql_iter_table(self):
        res = set()
        for it in z.mysql_iter_table(self.client, table="sca.t_std_org", where_condition="is_deleted=0", offset_limit="limit 10"):
            res.add(it["code"])
        self.assertEqual(len(res), 10)
        
        for it in z.mysql_iter_table(self.client, table="sca.t_std_org", fields={"code"}, where_condition="is_deleted=0", offset_limit="limit 10"):
            self.assertSetEqual(set(it.keys()), {"code"})
        
        for it in z.mysql_iter_table(self.client, table="sca.t_std_org", fields={"code", "name"}, where_condition="is_deleted=0", offset_limit="limit 10"):
            self.assertSetEqual(set(it.keys()), {"code", "name"})


import unittest


class ZMongoTests(unittest.TestCase):
    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import remove_dir
        remove_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_mongo_config(self):
        from zzpy import MongoConfig

        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4"),
                         MongoConfig(host="1.2.3.4", port=27017))
        
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4/"),
                         MongoConfig(host="1.2.3.4", port=27017))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4:5"),
                         MongoConfig(host="1.2.3.4", port=5))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4:5/d"),
                         MongoConfig(host="1.2.3.4", port=5, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4/d"),
                         MongoConfig(host="1.2.3.4", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4/d?"),
                         MongoConfig(host="1.2.3.4", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4:5/d?x=1"),
                         MongoConfig(host="1.2.3.4", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="mongodb://1.2.3.4:5/d?x=1&y=2"),
                         MongoConfig(host="1.2.3.4", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MongoConfig(url="mongodb://localhost"),
                         MongoConfig(host="localhost", port=27017))
        self.assertEqual(MongoConfig(url="mongodb://localhost/"),
                         MongoConfig(host="localhost", port=27017))
        self.assertEqual(MongoConfig(url="mongodb://localhost:5"),
                         MongoConfig(host="localhost", port=5))
        self.assertEqual(MongoConfig(url="mongodb://localhost:5/d"),
                         MongoConfig(host="localhost", port=5, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://localhost/d"),
                         MongoConfig(host="localhost", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://localhost/d?"),
                         MongoConfig(host="localhost", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://localhost:5/d?x=1"),
                         MongoConfig(host="localhost", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="mongodb://localhost:5/d?x=1&y=2"),
                         MongoConfig(host="localhost", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MongoConfig(url="mongodb://a"),
                         MongoConfig(host="a", port=27017))
        self.assertEqual(MongoConfig(url="mongodb://a/"),
                         MongoConfig(host="a", port=27017))
        self.assertEqual(MongoConfig(url="mongodb://a:5"),
                         MongoConfig(host="a", port=5))
        self.assertEqual(MongoConfig(url="mongodb://a:5/d"),
                         MongoConfig(host="a", port=5, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://a/d"),
                         MongoConfig(host="a", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://a/d?"),
                         MongoConfig(host="a", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="mongodb://a:5/d?x=1"),
                         MongoConfig(host="a", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="mongodb://a:5/d?x=1&y=2"),
                         MongoConfig(host="a", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MongoConfig(url="1.2.3.4"),
                         MongoConfig(host="1.2.3.4", port=27017))
        self.assertEqual(MongoConfig(url="1.2.3.4/"),
                         MongoConfig(host="1.2.3.4", port=27017))
        self.assertEqual(MongoConfig(url="1.2.3.4:5"),
                         MongoConfig(host="1.2.3.4", port=5))
        self.assertEqual(MongoConfig(url="1.2.3.4:5/d"),
                         MongoConfig(host="1.2.3.4", port=5, database="d"))
        self.assertEqual(MongoConfig(url="1.2.3.4/d"),
                         MongoConfig(host="1.2.3.4", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="1.2.3.4/d?"),
                         MongoConfig(host="1.2.3.4", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="1.2.3.4:5/d?x=1"),
                         MongoConfig(host="1.2.3.4", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="1.2.3.4:5/d?x=1&y=2"),
                         MongoConfig(host="1.2.3.4", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MongoConfig(url="localhost"),
                         MongoConfig(host="localhost", port=27017))
        self.assertEqual(MongoConfig(url="localhost/"),
                         MongoConfig(host="localhost", port=27017))
        self.assertEqual(MongoConfig(url="localhost:5"),
                         MongoConfig(host="localhost", port=5))
        self.assertEqual(MongoConfig(url="localhost:5/d"),
                         MongoConfig(host="localhost", port=5, database="d"))
        self.assertEqual(MongoConfig(url="localhost/d"),
                         MongoConfig(host="localhost", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="localhost/d?"),
                         MongoConfig(host="localhost", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="localhost:5/d?x=1"),
                         MongoConfig(host="localhost", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="localhost:5/d?x=1&y=2"),
                         MongoConfig(host="localhost", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MongoConfig(url="a"),
                         MongoConfig(host="a", port=27017))
        self.assertEqual(MongoConfig(url="a/"),
                         MongoConfig(host="a", port=27017))
        self.assertEqual(MongoConfig(url="a:5"),
                         MongoConfig(host="a", port=5))
        self.assertEqual(MongoConfig(url="a:5/d"),
                         MongoConfig(host="a", port=5, database="d"))
        self.assertEqual(MongoConfig(url="a/d"),
                         MongoConfig(host="a", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="a/d?"),
                         MongoConfig(host="a", port=27017, database="d"))
        self.assertEqual(MongoConfig(url="a:5/d?x=1"),
                         MongoConfig(host="a", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MongoConfig(url="a:5/d?x=1&y=2"),
                         MongoConfig(host="a", port=5, database="d", param={"x": "1", "y": "2"}))



__MONGO_URL_KEY = "MONGO_URL"


class MongoConfig:
    default_host = "localhost"
    default_port = 27017

    def __init__(self, url=None, host=None, port=None, database=None, param=None):
        import re
        if url:
            result = re.search(
                "^(mongodb://){0,1}([^:/]+)(:(\d+)){0,1}(/([^?]+)){0,1}(\?(.*)){0,1}", url)
            groups = result.groups()
            if not host:
                host = groups[1] if groups[1] else self.default_host
            if not port:
                port = int(groups[3]) if groups[3] else self.default_port
            if not database:
                database = groups[5] if groups[5] else None
            if not param:
                param = dict([it.split('=')
                              for it in groups[7].split('&')]) if groups[7] else {}
        self.url = url
        self.host = host
        self.port = port
        self.database = database
        self.param = param if param else {}

    def __eq__(self, value):
        return self.host == value.host and self.port == value.port and self.database == value.database and self.param == value.param


def mongo_connect(url=None):
    if not url:
        from .zconfig import get_param
        url = get_param(__MONGO_URL_KEY)
    assert url

    from pymongo import MongoClient
    import pytz

    conf = MongoConfig(url)
    client = MongoClient(host=conf.host, port=conf.port, tz_aware=True, tzinfo=pytz.timezone(
        'Asia/Shanghai'))
    if conf.database:
        client = client.get_database(conf.database)
    return client


def main():
    assert MongoConfig(url="mongodb://a:1") == MongoConfig(host="a", port=1)


if __name__ == "__main__":
    main()



def pb(iterable, total=None, title='进度', min_interval=10, max_interval=60, ncols=80):
    if not total:
        try:
            total = len(iterable)
        except Exception as ex:
            pass
        
    from tqdm import tqdm
    return tqdm(iterable,
                total=total,
                desc=f'\n{title}',
                mininterval=min_interval,
                maxinterval=max_interval,
                ncols=ncols)


def main():
    import time
    for i in pb(range(500), title='任务'):
        time.sleep(0.02)


if __name__ == "__main__":
    main()



class OssConfig:
    def __init__(self, access_key_id='', access_key_secret='', bucket='', endpoint=''):
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.bucket = bucket
        self.endpoint = endpoint


class OssFile:
    def __init__(self, config=None):
        import oss2
        if not config:
            from .zconfig import get_param
            config = OssConfig()
            config.access_key_id = get_param("OSS_ACCESS_KEY_ID", default_value='')
            config.access_key_secret = get_param(
                "OSS_ACCESS_KEY_SECRET", default_value='')
            config.bucket = get_param("OSS_BUCKET", default_value='')
            config.endpoint = get_param("OSS_ENDPOINT", default_value='')
        self.bucket = oss2.Bucket(
            oss2.Auth(config.access_key_id, config.access_key_secret), config.endpoint, config.bucket)

    def download(self, key, file_path):
        """下载文件"""
        return self.bucket.get_object_to_file(key, file_path)

    def upload(self, key, file_path, infrequent_access_flag=True):
        """上传文件，infrequent_access_flag低频标识"""
        import oss2
        headers = {
            'x-oss-storage-class': oss2.BUCKET_STORAGE_CLASS_IA} if infrequent_access_flag else None
        return self.bucket.put_object_from_file(key, file_path)

    def delete(self, prefix):
        key_list = [it.key for it in self.bucket.list_objects(
            prefix=prefix).object_list]
        if len(key_list) > 0:
            return self.bucket.batch_delete_objects(key_list=key_list)
        return None

    def exist(self, key):
        """判断文件是否已存在"""
        return self.bucket.object_exists(key)

    def not_exist(self, key):
        """判断文件是否不存在"""
        return not self.exist(key)

    def meta(self, key):
        """获取文件元信息"""
        return self.bucket.get_object_meta(key)



def randint(min=0, max=9):
    import random
    if min > max:
        min, max = max, min
    return random.randint(min, max)


def randhex(digits=1, prefix=False, lower=False):
    import random
    alpha = '0123456789ABCDEF'
    prefix_str = '0x' if prefix else ''
    num_str = ''.join(random.choice(alpha)
                      for _ in range(digits))
    if lower:
        num_str = num_str.lower()
    return prefix_str+num_str



DEFAULT_ROOT_DIR = "output"


def remove_dir(dir_path):
    """删除文件夹"""
    import os
    import shutil
    if os.path.exists(dir_path):
        try:
            os.removedirs(dir_path)
        except Exception as ex:
            shutil.rmtree(dir_path, ignore_errors=True)


def create_dir(dir_path):
    """创建文件夹"""
    import os
    import shutil
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)


def init_dir(dir_path):
    """初始化（先删除再创建）文件夹"""
    remove_dir(dir_path)
    create_dir(dir_path)


def get_one_file_path_from_dir(dir_path):
    """返回文件夹中（第一个）文件的地址"""
    import os
    for parent, dirnames, filenames in os.walk(dir_path):
        for filename in filenames:
            return os.path.join(parent, filename)
    return None


def move_file_from_src_dir(dst_file_path, src_dir_path):
    """从src_dir中拿到（唯一）文件，移动到dst_dir，并重命名"""
    import shutil
    import os
    src_file_path = get_one_file_path_from_dir(src_dir_path)
    if not src_file_path or not os.path.exists(src_file_path):
        return
    if not os.path.exists(os.path.dirname(dst_file_path)):
        create_dir(os.path.dirname(dst_file_path))
    shutil.move(src_file_path, dst_file_path)


def get_file_line_count(file_path):
    """获取文件行数"""
    count = 0
    with open(file_path, 'rb') as fr:
        for _ in enumerate(fr):
            count += 1
    return count


def get_file_name_list_from_dir(dir_path):
    """获取文件夹下所有文件名"""
    import os
    file_names = []
    for parent_dir, dirs, files in os.walk(dir_path):
        for file_name in files:
            file_names.append(file_name)
    return file_names


def get_file_path_list_from_dir(dir_path):
    """获取文件夹下所有文件路径"""
    import os
    file_pathes = []
    for parent_dir, dirs, files in os.walk(dir_path):
        for file_name in files:
            file_path = os.path.join(parent_dir, file_name)
            file_pathes.append(file_path)
    return file_pathes


def _read_file_using_encoding(file_path, encoding):
    """按编码读取文件"""
    f = None
    text = None
    try:
        f = open(file_path, encoding=encoding)
        text = f.read()
    except:
        pass
    finally:
        if f:
            f.close()
    return text


def read_bin_file(file_path):
    """读取二进制文件"""
    data = None
    with open(file_path, mode="rb") as fr:
        data = fr.read()
    return data


def read_file(file_path):
    """读取文件"""
    for encoding in ("utf8", "gbk"):
        text = _read_file_using_encoding(file_path, encoding=encoding)
        if text is not None:
            return text
    return None


def write_file(content, file_path, encoding=None):
    """写文件"""
    import os
    if encoding is None:
        encoding = "utf8"
    dir_path = os.path.dirname(file_path)
    if not os.path.exists(dir_path):
        create_dir(dir_path)
    with open(file_path, mode="w", encoding=encoding) as fw:
        fw.write(content)


def get_tmp_dir_path(root_dir=None):
    """临时目录"""
    if root_dir is None:
        root_dir = DEFAULT_ROOT_DIR
    import os
    return os.path.join(root_dir, "tmp")


def get_work_dir_path(root_dir=None):
    """工作目录"""
    if root_dir is None:
        root_dir = DEFAULT_ROOT_DIR
    import os
    return os.path.join(root_dir, "work")


def init_dirs(root_dir=None):
    '''初始化所有目录：临时目录、工作目录'''
    if root_dir is None:
        root_dir = DEFAULT_ROOT_DIR
    for dir_path_func in (get_tmp_dir_path, get_work_dir_path):
        dir_path = dir_path_func(root_dir=root_dir)
        init_dir(dir_path)


def read_jsonline(file_path):
    """从jsonline文件按行读取"""
    import jsonlines
    with open(file_path, encoding='utf8') as fr:
        yield from jsonlines.Reader(fr)


def read_jsonline_with_progressbar(file_path, title=None):
    """从jsonline文件按行读取，带进度条"""
    import jsonlines
    from zzpy import pb
    with open(file_path, encoding='utf8') as fr:
        yield from pb(jsonlines.Reader(fr), total=get_file_line_count(file_path), title=title)


def _split_path_list(path_list, split_pattern):
    component_list = []
    for path in path_list:
        component_list.extend(path.split(split_pattern))
    return component_list


def normalize_path(path):
    """标准化路径"""
    import os
    components = [path]
    for split_pattern in ("/", "\\"):
        components = _split_path_list(components, split_pattern=split_pattern)
    prefix = "/" if path.startswith("/") else ""
    return prefix + os.path.join(*list(components))
